from typing import TYPE_CHECKING

from sqlalchemy import Column, Integer, String
from sqlalchemy.orm import relationship
from beastiary.db.base_class import Base

if TYPE_CHECKING:
    from beastiary.models.sample import Sample


class Trace(Base):
    id = Column(Integer, primary_key=True)
    path = Column(String)
    headers_line = Column(String, default="")
    last_byte = Column(Integer, default=0)
    samples = relationship("Sample", back_populates="trace")



from pydantic import BaseModel
from pathlib import Path
from typing import Optional


# Shared properties
class TraceBase(BaseModel):
    path: Path


# Properties to receive on Trace creation
class TraceCreate(TraceBase):
    pass


# Properties to receive on Trace update
class TraceUpdate(TraceBase):
    pass


# Properties shared by models stored in DB
class TraceInDBBase(TraceBase):
    id: int
    headers_line: str
    last_byte: int

    class Config:
        orm_mode = True


# Properties to return to client
class Trace(TraceInDBBase):
    pass


# Properties properties stored in DB
class TraceInDB(TraceInDBBase):
    pass



from typing import Optional

from pydantic import BaseModel


# Shared properties
class SampleBase(BaseModel):
    state: Optional[int] = None
    data: Optional[dict] = None


# Properties to receive on Sample creation
class SampleCreate(SampleBase):
    state: int
    data: dict


# Properties to receive on Sample update
class SampleUpdate(SampleBase):
    pass


# Properties shared by models stored in DB
class SampleInDBBase(SampleBase):
    id: int
    trace_id: int

    class Config:
        orm_mode = True


# Properties to return to client
class Sample(SampleInDBBase):
    pass



from sqlalchemy import create_engine
from sqlalchemy.orm import sessionmaker
from sqlalchemy.pool import StaticPool


engine = create_engine(
    "sqlite://",
    pool_pre_ping=True,
    poolclass=StaticPool,
    connect_args={"check_same_thread": False},
)
SessionLocal = sessionmaker(autocommit=False, autoflush=False, bind=engine)



from typing import Any

from sqlalchemy.ext.declarative import as_declarative, declared_attr


@as_declarative()
class Base:
    id: Any
    __name__: str
    # Generate __tablename__ automatically
    @declared_attr
    def __tablename__(cls) -> str:
        return cls.__name__.lower()



from sqlalchemy.orm import Session
from beastiary.db import base  # noqa: F401
from .session import engine

# make sure all SQL Alchemy models are imported (app.db.base) before initializing DB
# otherwise, SQL Alchemy might fail to initialize relationships properly
# for more details: https://github.com/tiangolo/full-stack-fastapi-postgresql/issues/28


def init_db(db: Session) -> None:
    base.Base.metadata.create_all(bind=engine)
